@@ -1,12 +1,13 @@
 # coding: utf-8
 """Honeybee Room."""
 from ._basewithshade import _BaseWithShade
+from .typing import clean_string
 from .properties import RoomProperties
 from .face import Face
 from .facetype import get_type_from_normal, Wall, Floor
 from .boundarycondition import get_bc_from_position, Outdoors, Surface
-from .typing import float_in_range
-import honeybee.writer as writer
+from .typing import float_in_range, int_in_range
+import honeybee.writer.room as writer
 
 from ladybug_geometry.geometry2d.pointvector import Vector2D
 from ladybug_geometry.geometry3d.pointvector import Vector3D, Point3D
@@ -19,47 +20,52 @@ import math
 class Room(_BaseWithShade):
     """A volume enclosed by faces, representing a single room or space.
 
-    Properties:
-        name
-        display_name
-        faces
-        indoor_shades
-        outdoor_shades
-        geometry
-        center
-        volume
-        floor_area
-        exposed_area
-        exterior_wall_area
-        exterior_aperture_area
-        average_floor_height
-    """
-    __slots__ = ('_geometry', '_faces')
+    Args:
+        identifier: Text string for a unique Room ID. Must be < 100 characters and
+            not contain any spaces or special characters.
+        faces: A list or tuple of honeybee Face objects that together form the
+            closed volume of a room.
+        tolerance: The maximum difference between x, y, and z values
+            at which vertices of adjacent faces are considered equivalent. This is
+            used in determining whether the faces form a closed volume. Default
+            is 0, which makes no attempt to evaluate whether the Room volume
+            is closed.
+        angle_tolerance: The max angle difference in degrees that vertices are
+            allowed to differ from one another in order to consider them colinear.
+            Default is 0, which makes no attempt to evaluate whether the Room
+            volume is closed.
 
-    def __init__(self, name, faces, tolerance=None, angle_tolerance=None):
+    Properties:
+        * identifier
+        * display_name
+        * faces
+        * multiplier
+        * indoor_furniture
+        * indoor_shades
+        * outdoor_shades
+        * geometry
+        * center
+        * volume
+        * floor_area
+        * exposed_area
+        * exterior_wall_area
+        * exterior_aperture_area
+        * average_floor_height
+        * user_data
+    """
+    __slots__ = ('_geometry', '_faces', '_multiplier')
+
+    def __init__(self, identifier, faces, tolerance=0, angle_tolerance=0):
         """A volume enclosed by faces, representing a single room or space.
 
-        Note that, if None is input for tolerance and angle_tolerance, no checks will
-        be performed to determine whether the room is a closed volume and no attempt
-        will be made to flip faces in the event that they are not facing outward from
-        the room volume.  As such, an input tolerance of None is intended for
-        workflows where the solidity of the room volume has been evaluated elsewhere.
-
-        Args:
-            name: Room name. Must be < 100 characters.
-            faces: A list or tuple of honeybee Face objects that together form the
-                closed volume of a room.
-            tolerance: tolerance: The maximum difference between x, y, and z values
-                at which vertices of adjacent facesare considered equivalent. This is
-                used in determining whether the faces form a closed volume. Default
-                is None, which makes no attempt to evaluate whether the Room volume
-                is closed.
-            angle_tolerance: The max angle difference in degrees that vertices are
-                allowed to differ from one another in order to consider them colinear.
-                Default is None, which makes no attempt to evaluate whether the Room
-                volume is closed.
+        Note that, if zero is input for tolerance and angle_tolerance, no checks
+        will be performed to determine whether the room is a closed volume
+        and no attempt will be made to flip faces in the event that they are not
+        facing outward from the room volume.  As such, an input tolerance of 0
+        is intended for workflows where the solidity of the room volume has been
+        evaluated elsewhere.
         """
-        _BaseWithShade.__init__(self, name)  # process the name
+        _BaseWithShade.__init__(self, identifier)  # process the identifier
 
         # process the zone volume geometry
         if not isinstance(faces, tuple):
@@ -69,14 +75,14 @@ class Room(_BaseWithShade):
                 'Expected honeybee Face. Got {}'.format(type(face))
             face._parent = self
 
-        if tolerance is None:
+        if tolerance == 0:
             self._faces = faces
             self._geometry = None  # calculated later from faces or added by classmethods
         else:
             # try to get a closed volume between the faces
             room_polyface = Polyface3D.from_faces(
                 tuple(face.geometry for face in faces), tolerance)
-            if not room_polyface.is_solid and angle_tolerance is not None:
+            if not room_polyface.is_solid and angle_tolerance != 0:
                 ang_tol = math.radians(angle_tolerance)
                 room_polyface = room_polyface.merge_overlapping_edges(tolerance, ang_tol)
             # replace honeybee face geometry with versions that are facing outwards
@@ -86,33 +92,51 @@ class Room(_BaseWithShade):
             self._faces = faces
             self._geometry = room_polyface
 
-        # initialize properties for extensions
-        self._properties = RoomProperties(self)
+        self._multiplier = 1  # default value that can be overridden later
+        self._properties = RoomProperties(self)  # properties for extensions
 
     @classmethod
-    def from_dict(cls, data):
+    def from_dict(cls, data, tolerance=0, angle_tolerance=0):
         """Initialize an Room from a dictionary.
 
         Args:
-            data: A dictionary representation of an Room object.
+            data: A dictionary representation of a Room object.
+            tolerance: The maximum difference between x, y, and z values
+                at which vertices of adjacent faces are considered equivalent. This is
+                used in determining whether the faces form a closed volume. Default
+                is 0, which makes no attempt to evaluate whether the Room volume
+                is closed.
+            angle_tolerance: The max angle difference in degrees that vertices are
+                allowed to differ from one another in order to consider them colinear.
+                Default is 0, which makes no attempt to evaluate whether the Room
+                volume is closed.
         """
         # check the type of dictionary
         assert data['type'] == 'Room', 'Expected Room dictionary. ' \
             'Got {}.'.format(data['type'])
 
-        room = Room(data['name'], [Face.from_dict(f_dict) for f_dict in data['faces']])
+        room = cls(data['identifier'], [Face.from_dict(f_dict) for f_dict in data['faces']],
+                   tolerance, angle_tolerance)
         if 'display_name' in data and data['display_name'] is not None:
-            room._display_name = data['display_name']
+            room.display_name = data['display_name']
+        if 'user_data' in data and data['user_data'] is not None:
+            room.user_data = data['user_data']
+        if 'multiplier' in data and data['multiplier'] is not None:
+            room._multiplier = data['multiplier']
         room._recover_shades_from_dict(data)
+
+        if data['properties']['type'] == 'RoomProperties':
+            room.properties._load_extension_attr_from_dict(data['properties'])
         return room
 
     @classmethod
-    def from_polyface3d(cls, name, polyface, roof_angle=30, floor_angle=150,
+    def from_polyface3d(cls, identifier, polyface, roof_angle=30, floor_angle=150,
                         ground_depth=0):
         """Initialize a Room from a ladybug_geometry Polyface3D object.
 
         Args:
-            name: Room name. Must be < 100 characters.
+            identifier: Text string for a unique Room ID. Must be < 100 characters and
+                not contain any spaces or special characters.
             polyface: A ladybug_geometry Polyface3D object representing the closed
                 volume of a room. The Polyface3D.is_solid property can be used to
                 determine whether the polyface is a closed solid before input here.
@@ -120,21 +144,21 @@ class Room(_BaseWithShade):
             floor_angle: Cutting angle for floor from Z axis in degrees. Default: 150.
             ground_depth: The Z value above which faces are considered Outdoors
                 instead of Ground. Faces will have a Ground boundary condition if
-                their center point lies at or below this value. Default: 0.
+                all of their vertices lie at or below this value. Default: 0.
         """
         assert isinstance(polyface, Polyface3D), \
             'Expected ladybug_geometry Polyface3D. Got {}'.format(type(polyface))
         faces = []
         for i, face in enumerate(polyface.faces):
-            faces.append(Face('{}_Face{}'.format(name, i), face,
+            faces.append(Face('{}..Face{}'.format(identifier, i), face,
                               get_type_from_normal(face.normal, roof_angle, floor_angle),
                               get_bc_from_position(face.boundary, ground_depth)))
-        room = cls(name, faces)
+        room = cls(identifier, faces)
         room._geometry = polyface
         return room
 
     @classmethod
-    def from_box(cls, name, width=3.0, depth=6.0, height=3.2,
+    def from_box(cls, identifier, width=3.0, depth=6.0, height=3.2,
                  orientation_angle=0, origin=Point3D(0, 0, 0)):
         """Initialize a Room from parameters describing a box.
 
@@ -143,7 +167,8 @@ class Room(_BaseWithShade):
         cardinal direction of the orientation_angle.
 
         Args:
-            name: Room name. Must be < 100 characters.
+            identifier: Text string for a unique Room ID. Must be < 100 characters and
+                not contain any spaces or special characters.
             width: Number for the width of the box (in the X direction). Default: 3.0.
             depth: Number for the depth of the box (in the Y direction). Default: 6.0.
             height: Number for the height of the box (in the Z direction). Default: 3.2.
@@ -165,21 +190,50 @@ class Room(_BaseWithShade):
         directions = ('Bottom', 'Front', 'Right', 'Back', 'Left', 'Top')
         faces = []
         for face, dir in zip(polyface.faces, directions):
-            faces.append(Face('{}_{}'.format(name, dir), face,
+            faces.append(Face('{}_{}'.format(identifier, dir), face,
                               get_type_from_normal(face.normal),
                               get_bc_from_position(face.boundary)))
-        room = cls(name, faces)
+        room = cls(identifier, faces)
         room._geometry = polyface
         return room
 
     @property
     def faces(self):
-        """Tuple of all honeybee Faces making up this room volume."""
+        """Get a tuple of all honeybee Faces making up this room volume."""
         return self._faces
 
+    @property
+    def multiplier(self):
+        """Get or set an integer noting how many times this Room is repeated.
+
+        Multipliers are used to speed up the calculation when similar Rooms are
+        repeated more than once. Essentially, a given simulation with the
+        Room is run once and then the result is mutliplied by the multiplier.
+        This means that the "repetition" isn't in a particular direction (it's
+        essentially in the exact same location) and this comes with some
+        inaccuracy. However, this error might not be too large if the Rooms
+        are similar enough and it can often be worth it since it can greatly
+        speed up the calculation.
+        """
+        return self._multiplier
+
+    @multiplier.setter
+    def multiplier(self, value):
+        self._multiplier = int_in_range(value, 1, input_name='room multiplier')
+
+    @property
+    def indoor_furniture(self):
+        """Array of all indoor furniture Shade objects assigned to this Room.
+
+        Note that this property is identical to the indoor_shades property but
+        it is provided here under an alternate name to make it clear that indoor
+        furniture objects should be added here to the Room.
+        """
+        return tuple(self._indoor_shades)
+
     @property
     def geometry(self):
-        """A ladybug_geometry Polyface3D object representing the room."""
+        """Get a ladybug_geometry Polyface3D object representing the room."""
         if self._geometry is None:
             self._geometry = Polyface3D.from_faces(
                 tuple(face.geometry for face in self._faces))
@@ -187,7 +241,7 @@ class Room(_BaseWithShade):
 
     @property
     def center(self):
-        """A ladybug_geometry Point3D for the center of the room.
+        """Get a ladybug_geometry Point3D for the center of the room.
 
         Note that this is the center of the bounding box around the room geometry
         and not the volume centroid.
@@ -196,7 +250,7 @@ class Room(_BaseWithShade):
 
     @property
     def volume(self):
-        """The volume of the room.
+        """Get the volume of the room.
 
         Note that, if this room faces do not form a closed solid (with all face normals
         pointing outward), the value of this property will not be accurate.
@@ -205,12 +259,12 @@ class Room(_BaseWithShade):
 
     @property
     def floor_area(self):
-        """The combined area of all room floor faces."""
+        """Get the combined area of all room floor faces."""
         return sum([face.area for face in self._faces if isinstance(face.type, Floor)])
 
     @property
     def exposed_area(self):
-        """The combined area of all room faces with outdoor boundary conditions.
+        """Get the combined area of all room faces with outdoor boundary conditions.
 
         Useful for estimating infiltration, often expressed as a flow per
         unit exposed envelope area.
@@ -220,7 +274,7 @@ class Room(_BaseWithShade):
 
     @property
     def exterior_wall_area(self):
-        """The combined area of all exterior walls on the room.
+        """Get the combined area of all exterior walls on the room.
 
         Useful for calculating glazing ratios.
         """
@@ -233,7 +287,7 @@ class Room(_BaseWithShade):
 
     @property
     def exterior_aperture_area(self):
-        """The combined area of all exterior apertures on the room.
+        """Get the combined area of all exterior apertures on the room.
 
         Useful for calculating glazing ratios.
         """
@@ -245,7 +299,7 @@ class Room(_BaseWithShade):
 
     @property
     def average_floor_height(self):
-        """The height of the room floor averaged over all floor faces in the room.
+        """Get the height of the room floor averaged over all floor faces in the room.
 
         Will be None if the room posseses no floors. Resulting value is weighted by
         the area of each of the floor faces.
@@ -264,9 +318,9 @@ class Room(_BaseWithShade):
         return False
 
     def average_orientation(self, north_vector=Vector2D(0, 1)):
-        """A number between 0 and 360 for the average orientation of exposed walls.
+        """Get a number between 0 and 360 for the average orientation of exposed walls.
 
-        0 = North, 90 = East, 180 = South, 270 = West.  Wil be None if the zone has
+        0 = North, 90 = East, 180 = South, 270 = West.  Will be None if the zone has
         no exterior walls. Resulting value is weighted by the area of each of the
         wall faces.
 
@@ -283,6 +337,50 @@ class Room(_BaseWithShade):
                 areas += face.area
         return orientations / areas if areas != 0 else None
 
+    def add_prefix(self, prefix):
+        """Change the identifier of this object and all child objects by inserting a prefix.
+
+        This is particularly useful in workflows where you duplicate and edit
+        a starting object and then want to combine it with the original object
+        into one Model (like making a model of repeated rooms) since all objects
+        within a Model must have unique identifiers.
+
+        Args:
+            prefix: Text that will be inserted at the start of this object's
+                (and child objects') identifier and display_name. It is recommended
+                that this prefix be short to avoid maxing out the 100 allowable
+                characters for honeybee identifiers.
+        """
+        self._identifier = clean_string('{}_{}'.format(prefix, self.identifier))
+        self.display_name = '{}_{}'.format(prefix, self.display_name)
+        self.properties.add_prefix(prefix)
+        for face in self._faces:
+            face.add_prefix(prefix)
+        self._add_prefix_shades(prefix)
+
+
+    def remove_indoor_furniture(self):
+        """Remove all indoor furniture assigned to this Room.
+
+        Note that this method is identical to the remove_indoor_shade method but
+        it is provided here under an alternate name to make it clear that indoor
+        furniture objects should be added here to the Room.
+        """
+        self.remove_indoor_shades()
+
+    def add_indoor_furniture(self, shade):
+        """Add a Shade object representing furniture to the Room.
+
+        Note that this method is identical to the add_indoor_shade method but
+        it is provided here under an alternate name to make it clear that indoor
+        furniture objects should be added here to the Room.
+
+        Args:
+            shade: A Shade object to add to the indoors of this Room, representing
+                furniture, desks, partitions, etc.
+        """
+        self.add_indoor_shade(shade)
+
     def generate_grid(self, x_dim, y_dim=None, offset=1.0):
         """Get a list of gridded Mesh3D objects offset from the floors of this room.
 
@@ -299,6 +397,9 @@ class Room(_BaseWithShade):
                 the floor.
 
         Usage:
+
+        .. code-block:: python
+
             room = Room.from_box(3.0, 6.0, 3.2, 180)
             floor_mesh = room.generate_mesh_grid(0.5, 0.5, 1)
             test_points = floor_mesh[0].face_centroids
@@ -306,7 +407,7 @@ class Room(_BaseWithShade):
         floor_grids = []
         for face in self._faces:
             if isinstance(face.type, Floor):
-                floor_grids.append(face.geometry.get_mesh_grid(
+                floor_grids.append(face.geometry.mesh_grid(
                     x_dim, y_dim, offset, True))
         return floor_grids
 
@@ -320,6 +421,7 @@ class Room(_BaseWithShade):
         for face in self._faces:
             face.move(moving_vec)
         self.move_shades(moving_vec)
+        self.properties.move(moving_vec)
         if self._geometry is not None:
             self._geometry = self.geometry.move(moving_vec)
 
@@ -335,6 +437,7 @@ class Room(_BaseWithShade):
         for face in self._faces:
             face.rotate(axis, angle, origin)
         self.rotate_shades(axis, angle, origin)
+        self.properties.rotate(axis, angle, origin)
         if self._geometry is not None:
             self._geometry = self.geometry.rotate(axis, math.radians(angle), origin)
 
@@ -349,6 +452,7 @@ class Room(_BaseWithShade):
         for face in self._faces:
             face.rotate_xy(angle, origin)
         self.rotate_xy_shades(angle, origin)
+        self.properties.rotate_xy(angle, origin)
         if self._geometry is not None:
             self._geometry = self.geometry.rotate_xy(math.radians(angle), origin)
 
@@ -362,6 +466,7 @@ class Room(_BaseWithShade):
         for face in self._faces:
             face.reflect(plane)
         self.reflect_shades(plane)
+        self.properties.reflect(plane)
         if self._geometry is not None:
             self._geometry = self.geometry.reflect(plane.n, plane.o)
 
@@ -376,19 +481,23 @@ class Room(_BaseWithShade):
         for face in self._faces:
             face.scale(factor, origin)
         self.scale_shades(factor, origin)
+        self.properties.scale(factor, origin)
         if self._geometry is not None:
             self._geometry = self.geometry.scale(factor, origin)
 
-    def check_solid(self, tolerance, angle_tolerance, raise_exception=True):
+    def check_solid(self, tolerance=0.01, angle_tolerance=1, raise_exception=True):
         """Check whether the Room is a closed solid to within the input tolerances.
 
-        tolerance: tolerance: The maximum difference between x, y, and z values
-            at which face vertices are considered equivalent. This is used in
-            determining whether the faces form a closed volume.
-        angle_tolerance: The max angle difference in degrees that vertices are
-            allowed to differ from one another in order to consider them colinear.
-        raise_exception: Boolean to note whether a ValueError should be raised
-            if the room geometry does not form a closed solid.
+        Args:
+            tolerance: tolerance: The maximum difference between x, y, and z values
+                at which face vertices are considered equivalent. This is used in
+                determining whether the faces form a closed volume. Default: 0.01,
+                suitable for objects in meters.
+            angle_tolerance: The max angle difference in degrees that vertices are
+                allowed to differ from one another in order to consider them colinear.
+                Default: 1 degree.
+            raise_exception: Boolean to note whether a ValueError should be raised
+                if the room geometry does not form a closed solid.
         """
         if self._geometry is not None and self.geometry.is_solid:
             return True
@@ -406,14 +515,15 @@ class Room(_BaseWithShade):
                 'tolerance.'.format(self.display_name, tolerance, angle_tolerance))
         return False
 
-    def check_planar(self, tolerance, raise_exception=True):
+    def check_planar(self, tolerance=0.01, raise_exception=True):
         """Check that all of the Room's geometry components are planar.
 
         This includes all of the Room's Faces, Apertures, Doors and Shades.
 
         Args:
             tolerance: The minimum distance between a given vertex and a the
-                object's's plane at which the vertex is said to lie in the plane.
+                object's plane at which the vertex is said to lie in the plane.
+                Default: 0.01, suitable for objects in meters.
             raise_exception: Boolean to note whether an ValueError should be
                 raised if a vertex does not lie within the object's plane.
         """
@@ -464,7 +574,7 @@ class Room(_BaseWithShade):
         Args:
             tolerance: The minimum acceptable area of the object. Default is 0.0001,
                 which is equal to 1 cm2 when model units are meters. This is just
-                above the smalest size that OpenStudio will accept.
+                above the smallest size that OpenStudio will accept.
             raise_exception: If True, a ValueError will be raised if the object
                 area is below the tolerance. Default: True.
         """
@@ -484,26 +594,56 @@ class Room(_BaseWithShade):
         return self._check_non_zero_shades(tolerance, raise_exception)
 
     @staticmethod
-    def solve_adjcency(rooms, tolerance):
+    def solve_adjacency(rooms, tolerance=0.01):
         """Solve for all adjacencies between a list of input rooms.
 
         Args:
             rooms: A list of rooms for which adjacencies will be solved.
             tolerance: The minimum difference between the coordinate values of two
-                faces at which they can be considered centered adjacent.
+                faces at which they can be considered centered adjacent. Default: 0.01,
+                suitable for objects in meters.
+
+        Returns:
+            A dictionary of adjacency information with the following keys.
+
+            -   adjacent_faces - A list of tuples with each tuple containing 2 objects
+                for Faces paired in the process of solving adjacency. This data can
+                be used to assign custom properties to the new adjacent Faces (like
+                making all adjacencies an AirBoundary face type or assigning custom
+                materials/constructions).
+
+            -   adjacent_apertures - A list of tuples with each tuple containing 2
+                objects for Apertures paired in the process of solving adjacency.
+
+            -   adjacent_doors - A list of tuples with each tuple containing 2 objects
+                for Doors paired in the process of solving adjacency.
         """
+        # lists of adjacencies to track
+        adj_info = {'adjacent_faces': [], 'adjacent_apertures': [],
+                    'adjacent_doors': []}
+
+        # solve all adjacencies between rooms
         for i, room_1 in enumerate(rooms):
             try:
                 for room_2 in rooms[i + 1:]:
+                    if not Polyface3D.overlapping_bounding_boxes(
+                            room_1.geometry, room_2.geometry, tolerance):
+                        continue  # no overlap in bounding box; adjacency impossible
                     for face_1 in room_1._faces:
                         for face_2 in room_2._faces:
                             if not isinstance(face_2.boundary_condition, Surface):
                                 if face_1.geometry.is_centered_adjacent(
                                         face_2.geometry, tolerance):
-                                    face_1.set_adjacency(face_2)
+                                    face_info = face_1.set_adjacency(face_2)
+                                    adj_info['adjacent_faces'].append((face_1, face_2))
+                                    adj_info['adjacent_apertures'].extend(
+                                        face_info['adjacent_apertures'])
+                                    adj_info['adjacent_doors'].extend(
+                                        face_info['adjacent_doors'])
                                     break
             except IndexError:
                 pass  # we have reached the end of the list of zones
+        return adj_info
 
     @property
     def to(self):
@@ -512,10 +652,12 @@ class Room(_BaseWithShade):
         Use this method to access Writer class to write the room in other formats.
 
         Usage:
+
+        .. code-block:: python
+
             room.to.idf(room) -> idf string.
             room.to.radiance(room) -> Radiance string.
         """
-        raise NotImplementedError('Room does not yet support writing to files.')
         return writer
 
     def to_dict(self, abridged=False, included_prop=None):
@@ -523,28 +665,33 @@ class Room(_BaseWithShade):
 
         Args:
             abridged: Boolean to note whether the extension properties of the
-                object (ie. construciton sets) should be included in detail
-                (False) or just referenced by name (True). Default: False.
+                object (ie. construction sets) should be included in detail
+                (False) or just referenced by identifier (True). Default: False.
             included_prop: List of properties to filter keys that must be included in
                 output dictionary. For example ['energy'] will include 'energy' key if
                 available in properties to_dict. By default all the keys will be
                 included. To exclude all the keys from extensions use an empty list.
         """
         base = {'type': 'Room'}
-        base['name'] = self.name
+        base['identifier'] = self.identifier
         base['display_name'] = self.display_name
         base['properties'] = self.properties.to_dict(abridged, included_prop)
-
         base['faces'] = [f.to_dict(abridged, included_prop) for f in self._faces]
         self._add_shades_to_dict(base, abridged, included_prop)
+        if self.multiplier != 1:
+            base['multiplier'] = self.multiplier
+        if self.user_data is not None:
+            base['user_data'] = self.user_data
         return base
 
     def __copy__(self):
-        new_r = Room(self.name, tuple(face.duplicate() for face in self._faces))
+        new_r = Room(self.identifier, tuple(face.duplicate() for face in self._faces))
         new_r._display_name = self.display_name
+        new_r._user_data = None if self.user_data is None else self.user_data.copy()
+        new_r._multiplier = self.multiplier
         self._duplicate_child_shades(new_r)
         new_r._geometry = self._geometry
-        new_r._properties.duplicate_extension_attr(self._properties)
+        new_r._properties._duplicate_extension_attr(self._properties)
         return new_r
 
     def __len__(self):
